#! /usr/bin/perl 

#########################################################################################
# Input: two files containing the protein position and COG information in .ptt format
# Output: a single file listing the position of all occurances of each COG in two 
#         chromsomes. 
# Output format: <COG_id> <positions in 1st chromosome> <positions in 2nd chromosome>, 
#		 tab-delimited. The positions are ":"-separated integers or float numbers.
##########################################################################################

# Open the files
open( CHRA, "<$ARGV[0]") || die "Can't open the file $ARGV[0]";
open( CHRB, "<$ARGV[1]") || die "Can't open the file $ARGV[1]";

%COGP_A = ();
%COGP_B = ();

while ( <CHRA> ) {
  if ( /[0-9]+\.\.[0-9]+/ ) {
    @line = split "\t";
    $location = $line[0];
    $COG = $line[7];
    if ( $COG =~ /COG/ ) {
      $location =~ s/^\s+//;
      $start = $1 if ( $location =~ /([0-9]+)..([0-9]+)/ ); 
      $COGP_A{ $COG } = $COGP_A{ $COG }.$start.":";
    }
  }
}

foreach ( keys( %COGP_A ) ) {
  chop $COGP_A{ $_ };
  #print $_, "\t", $COGP_A{ $_ }, "\n";
}

while ( <CHRB> ) {
  if ( /[0-9]+\.\.[0-9]+/ ) {
    @line = split "\t";
    $location = $line[0];
    $COG = $line[7];
    if ( $COG =~ /COG/ ) {
      $location =~ s/^\s+//;
      $start = $1 if ( $location =~ /([0-9]+)..([0-9]+)/ );
      $COGP_B{ $COG } = $COGP_B{ $COG }.$start.":";
    }
  }
}

foreach ( keys( %COGP_B ) ) {
  chop $COGP_B{ $_ };
}

# Computer the common COGs of two chromosomes
@common_COGs = ();
%count = ();
foreach $gene ( keys( %COGP_A ) ) { $count{ $gene }++ };
foreach $gene ( keys( %COGP_B ) ) {
  if ( $count{ $gene } >= 1 ) {
    push @common_COGs, $gene;
  }
}

#foreach ( @common_COGs ) {
#  print $_, "\n";
#}

# Output the results
printf "%s\t%s\t%s\n", "FAMILY_ID", "CHROMOSOME1", "CHROMOSOME2";
foreach ( @common_COGs ) {
  printf "%s\t%s\t%s\n", $_, $COGP_A{ $_ }, $COGP_B{ $_ };
}


